/*
 * FreeRTOS Bananapi M3 port
 * (c) 2020 Dr.-Ing. Ingo D. Rullhusen <d01c@freeshell.de>
 *
 * BPI M3 port is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * BPI M3 port is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FreeRTOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

MEMORY 
{
	sram_a1	: ORIGIN = 0x00000000, LENGTH = 32K
	sram_a2 : ORIGIN = 0x00044000, LENGTH = 64K
	sram_b  : ORIGIN = 0x00020000, LENGTH = 96K
	ram	: ORIGIN = 0x40000000, LENGTH = 2048M
}

__sram_a1__ = 0x00000000;
__sram_a2__ = 0x00044000;

__stack_end__ = 0xc0000000 - 4;

SECTIONS 
{
	startup :
	{
		__vba__ = .;
		KEEP( *(.startup) )
	} >ram

	.text :
	{
		*(.text)
                *(.text*)
		*(.rodata)
		*(.rodata*)
		*(.glue_7)
		*(.glue_7t)
	} >ram

        __end_of_text__ = .;


/* .ARM.exidx is sorted, so has to go in its own output section.  */
	PROVIDE_HIDDEN (__exidx_start = .);
	.ARM.exidx :
	{
		*(.ARM.exidx* .gnu.linkonce.armexidx.*)
	} >ram
	PROVIDE_HIDDEN (__exidx_end = .);

        . = ALIGN(4);
/* Thread Local Storage sections  */
	.tdata :
	{
		*(.tdata .tdata.*)
	} >ram
	.tbss :
	{
		*(.tbss .tbss.*)
	} >ram
	.preinit_array :
	{
  		PROVIDE_HIDDEN (__preinit_array_start = .);
		KEEP (*(.preinit_array))
		PROVIDE_HIDDEN (__preinit_array_end = .);
	} >ram
	.init_array :
	{
		PROVIDE_HIDDEN (__init_array_start = .);
		KEEP (*(.init_array))
		PROVIDE_HIDDEN (__init_array_end = .);
	} >ram
	.fini_array :
	{
		PROVIDE_HIDDEN (__fini_array_start = .);
		KEEP (*(.fini_array))
		PROVIDE_HIDDEN (__fini_array_end = .);
	} >ram


        . = ALIGN(4);
	.data : 
	{
		__data_beg_src__ = .;
		__data_beg__ = __data_beg_src__; /* in place */
		*(.data)
                *(.data*)
	} >ram

       __data_end__ = .;

	. = ALIGN(4);
        .bss : 
        {
		__bss_beg__ = .;
                *(.bss)
        } >ram

	. = ALIGN(4);
	.eh_frame :
	{
		 KEEP (*(.eh_frame))
	} >ram

        _bss_end__ = . ; __bss_end__ = . ;



	/* Align here to ensure that the .bss section occupies space up to
	_end.  Align after .bss to ensure correct alignment even if the
	.bss section disappears because there are no input sections.  */
	. = ALIGN(32 / 8);

}

	. = ALIGN(32 / 8);
	_end = .;
	__end__ = . ;
	PROVIDE (end = .);

