#ifndef a83t_h
#define a83t_h
/*******************************************************************************
a83t.h - Register defs for Allwinner A83T

           
THE SOFTWARE IS DELIVERED "AS IS" WITHOUT WARRANTY OR CONDITION OF ANY KIND, 
EITHER EXPRESS, IMPLIED OR STATUTORY. THIS INCLUDES WITHOUT LIMITATION ANY 
WARRANTY OR CONDITION WITH RESPECT TO MERCHANTABILITY OR FITNESS FOR ANY 
PARTICULAR PURPOSE, OR AGAINST THE INFRINGEMENTS OF INTELLECTUAL PROPERTY RIGHTS 
OF OTHERS.
           
This file may be freely used for commercial and non-commercial applications, 
including being redistributed with any tools.

If you find a problem with the file, please report it so that it can be fixed.

Created by Sten Larsson (sten_larsson at yahoo com)

Edited by Richard Barry and Ingo D. Rullhusen.
*******************************************************************************/

#define REG8  (volatile unsigned char*)
#define REG16 (volatile unsigned short*)
#define REG32 (volatile unsigned int*)


/*##############################################################################
## MISC
##############################################################################*/

//
// CCU
#define CCU_BASE 0x01c20000
#define PLL_C0PUX_CTRL_REG (*(REG32 (CCU_BASE + 0x0000 )))
#define PLL_C1PUX_CTRL_REG (*(REG32 (CCU_BASE + 0x0000 )))

#define BUS_CLOCK_GATING_REG1   (*(REG32 (CCU_BASE + 0x0064)))
#define BUS_SOFTWARE_RESET_REG1 (*(REG32 (CCU_BASE + 0x02c4)))

//
// CPU Config

#define CPUCFG_BASE 0x01700000 //reg = <0x01700000 0x400>
#define C0_CTRL_REG0 (*(REG32 (CPUCFG_BASE + 0x0000 )))
#define C0_CTRL_REG1 (*(REG32 (CPUCFG_BASE + 0x0004 )))
#define C1_CTRL_REG0 (*(REG32 (CPUCFG_BASE + 0x0010 )))
#define C1_CTRL_REG1 (*(REG32 (CPUCFG_BASE + 0x0014 )))
#define C0_CPU_STATUS (*(REG32 (CPUCFG_BASE + 0x0030 )))
#define C1_CPU_STATUS (*(REG32 (CPUCFG_BASE + 0x0034 )))
#define C0_RST_CTRL (*(REG32 (CPUCFG_BASE + 0x0080 )))
#define C1_RST_CTRL (*(REG32 (CPUCFG_BASE + 0x0084 )))

//
// R PRCM

#define R_PRCM_BASE 0x01f01400 //reg = <0x01f01400 0x400>
#define C0_CPU_PO_RST_CTRL (*(REG32 (R_PRCM_BASE + 0x0004 )))
#define C1_CPU_PO_RST_CTRL (*(REG32 (R_PRCM_BASE + 0x0008 )))
#define C0CPUX_PWROFF_GATING_REG (*(REG32 (R_PRCM_BASE + 0x0100 )))
#define C1CPUX_PWROFF_GATING_REG (*(REG32 (R_PRCM_BASE + 0x0104 )))
#define C0_CPU0_PWR_SWITCH_CTRL (*(REG32 (R_PRCM_BASE + 0x0140 )))
#define C0_CPU1_PWR_SWITCH_CTRL (*(REG32 (R_PRCM_BASE + 0x0144 )))
#define C0_CPU2_PWR_SWITCH_CTRL (*(REG32 (R_PRCM_BASE + 0x0148 )))
#define C0_CPU3_PWR_SWITCH_CTRL (*(REG32 (R_PRCM_BASE + 0x014c )))
#define C1_CPU0_PWR_SWITCH_CTRL (*(REG32 (R_PRCM_BASE + 0x0150 )))
#define C1_CPU1_PWR_SWITCH_CTRL (*(REG32 (R_PRCM_BASE + 0x0154 )))
#define C1_CPU2_PWR_SWITCH_CTRL (*(REG32 (R_PRCM_BASE + 0x0158 )))
#define C1_CPU3_PWR_SWITCH_CTRL (*(REG32 (R_PRCM_BASE + 0x015c )))

//
// R CPU Config

#define R_CPUCFG_BASE 0x01f01c00 //reg = <0x1f01c00 0x400>
#define R_C0_PO_RST_CTRL (*(REG32 (R_CPUCFG_BASE + 0x30 )))
#define R_C1_PO_RST_CTRL (*(REG32 (R_CPUCFG_BASE + 0x34 )))
#define R_CPUCFG_CPU_SOFT_ENTRY_REG (*(REG32 (R_CPUCFG_BASE + 0x01a4 )))


//
// R Watchdog
#define RWDOG_BASE 0x01f01000
#define RWDOG_CTRL_REG (*(REG32 (RWDOG_BASE + 0x10 )))
#define RWDOG_CFG_REG  (*(REG32 (RWDOG_BASE + 0x14 )))
#define RWDOG_MODE_REG (*(REG32 (RWDOG_BASE + 0x18 )))


// IRQ Mask

#define IRQ_FIQ_MASK (*(REG32 (0x01700000 + 0x0040)))


// Spinlock

#define SPINLOCK_BASE 0x01c18000
#define SPLINLOCK_SYSTATUS_REG (*(REG32 (SPINLOCK_BASE + 0x0000)))
#define SPINLOCK_STATUS_REG    (*(REG32 (SPINLOCK_BASE + 0x0010)))
#define SPINLOCK_LOCK_REG(n)   (*(REG32 (SPINLOCK_BASE + 0x100+4*n)))


//
// GIC

#define GICC_CTLR  (*(REG32 (configINTERRUPT_CONTROLLER_BASE_ADDRESS+configINTERRUPT_CONTROLLER_INTERFACE_OFFSET)))
#define GICC_PMR   (*(REG32 (0x0004+configINTERRUPT_CONTROLLER_BASE_ADDRESS+configINTERRUPT_CONTROLLER_INTERFACE_OFFSET)))
#define GICC_BPR   (*(REG32 (0x0008+configINTERRUPT_CONTROLLER_BASE_ADDRESS+configINTERRUPT_CONTROLLER_INTERFACE_OFFSET)))
#define GICC_IAR   (*(REG32 (0x000c+configINTERRUPT_CONTROLLER_BASE_ADDRESS+configINTERRUPT_CONTROLLER_INTERFACE_OFFSET)))
#define GICC_EOIR  (*(REG32 (0x0010+configINTERRUPT_CONTROLLER_BASE_ADDRESS+configINTERRUPT_CONTROLLER_INTERFACE_OFFSET)))
#define GICC_AEOIR (*(REG32 (0x0024+configINTERRUPT_CONTROLLER_BASE_ADDRESS+configINTERRUPT_CONTROLLER_INTERFACE_OFFSET)))
#define GICC_IIDR  (*(REG32 (0x00fc+configINTERRUPT_CONTROLLER_BASE_ADDRESS+configINTERRUPT_CONTROLLER_INTERFACE_OFFSET)))
#define GICC_DIR   (*(REG32 (0x1000+configINTERRUPT_CONTROLLER_BASE_ADDRESS+configINTERRUPT_CONTROLLER_INTERFACE_OFFSET)))

#define GICD_CTLR         (*(REG32 (0x000+configINTERRUPT_CONTROLLER_BASE_ADDRESS+configINTERRUPT_CONTROLLER_DISTRIBUTOR_OFFSET)))
#define GICD_IGROUPR(n)   (*(REG32 (0x080+configINTERRUPT_CONTROLLER_BASE_ADDRESS+configINTERRUPT_CONTROLLER_DISTRIBUTOR_OFFSET+4*n)))
#define GICD_ISENABLER(n) (*(REG32 (0x100+configINTERRUPT_CONTROLLER_BASE_ADDRESS+configINTERRUPT_CONTROLLER_DISTRIBUTOR_OFFSET+4*n)))
#define GICD_ICENABLER(n) (*(REG32 (0x180+configINTERRUPT_CONTROLLER_BASE_ADDRESS+configINTERRUPT_CONTROLLER_DISTRIBUTOR_OFFSET+4*n)))
#define GICD_ICPENDR(n)   (*(REG32 (0x280+configINTERRUPT_CONTROLLER_BASE_ADDRESS+configINTERRUPT_CONTROLLER_DISTRIBUTOR_OFFSET+4*n)))
#define GICD_ICACTIVER(n) (*(REG32 (0x380+configINTERRUPT_CONTROLLER_BASE_ADDRESS+configINTERRUPT_CONTROLLER_DISTRIBUTOR_OFFSET+4*n)))
#define GICD_IPRIORITYR(n)(*(REG32 (0x400+configINTERRUPT_CONTROLLER_BASE_ADDRESS+configINTERRUPT_CONTROLLER_DISTRIBUTOR_OFFSET+4*n)))
#define GICD_ITARGETSR(n) (*(REG32 (0x800+configINTERRUPT_CONTROLLER_BASE_ADDRESS+configINTERRUPT_CONTROLLER_DISTRIBUTOR_OFFSET+4*n)))
#define GICD_SGIR         (*(REG32 (0xf00+configINTERRUPT_CONTROLLER_BASE_ADDRESS+configINTERRUPT_CONTROLLER_DISTRIBUTOR_OFFSET)))


//
// Reduced Serial Bus

#define RSB_BASE_ADDRESS (*(REG32 (0x01f03400)))
#define RSB_CTRL         (*(REG32 (0x01f03400 + 0x0000)))
#define RSB_CCR          (*(REG32 (0x01f03400 + 0x0004)))
#define RSB_INTE         (*(REG32 (0x01f03400 + 0x0008)))
#define RSB_STAT         (*(REG32 (0x01f03400 + 0x000c)))
#define RSB_DADDR0       (*(REG32 (0x01f03400 + 0x0010)))
#define RSB_DLEN         (*(REG32 (0x01f03400 + 0x0018)))
#define RSB_DATA0        (*(REG32 (0x01f03400 + 0x001c)))
#define RSB_LCR          (*(REG32 (0x01f03400 + 0x0024)))
#define RSB_PMCR         (*(REG32 (0x01f03400 + 0x0028)))
#define RSB_CMD          (*(REG32 (0x01f03400 + 0x002c)))
#define RSB_SADDR        (*(REG32 (0x01f03400 + 0x0030)))


/*##############################################################################
## PCB - Pin Connect Block
##############################################################################*/

#define PCB_PINSEL0     (*(REG32 (0xE002C000)))
#define PCB_PINSEL1     (*(REG32 (0xE002C004)))


/*##############################################################################
## GPIO - General Purpose I/O
##############################################################################*/

#define PB_CFG0_REG        (*(REG32 (0x01c20824)))
#define PB_DATA_REG        (*(REG32 ( 0x34+0x01c20800)))  // PB_DATA_REG
#define PB_PULL0_REG       (*(REG32 (0x01c20840)))
#define PB_EINT_CFG0_REG   (*(REG32 (0x200+0x01c20800)))
#define PB_EINT_CTL_REG    (*(REG32 (0x210+0x01c20800)))
#define PB_EINT_STATUS_REG (*(REG32 (0x214+0x01c20800)))

#define PC_CFG0_REG     (*(REG32 (0x01c20848)))
#define GPIO_IOSET      (*(REG32 (0x01c20858)))  // PC_DATA_REG


/*##############################################################################
## UART0 / UART1
##############################################################################*/

/* ---- UART 0 --------------------------------------------- */
#define UART0_RBR       (*(REG32 (0x01c28000)))
#define UART0_THR       (*(REG32 (0x01c28000)))
#define UART0_DLL       (*(REG32 (0x01c28000)))
#define UART0_DLM       (*(REG32 (0x01c28004))) // ?
#define UART0_IER       (*(REG32 (0x01c28004)))
#define UART0_IIR       (*(REG32 (0x01c28008)))
#define UART0_FCR       (*(REG32 (0x01c28008)))
#define UART0_LCR       (*(REG32 (0x01c2800C)))
#define UART0_LSR       (*(REG32 (0x01c28014)))
#define UART0_MSR       (*(REG32 (0x01c28018)))
#define UART0_SCH       (*(REG32 (0x01c2801C)))
#define UART0_USR       (*(REG32 (0x01c2807c)))
#define UART0_TFL       (*(REG32 (0x01c28080)))
#define UART0_RFL       (*(REG32 (0x01c28084)))
#define UART0_HALT      (*(REG32 (0x01c280a4)))

/* ---- UART 1 --------------------------------------------- */
#define UART1_RBR       (*(REG32 (0x01c28400)))
#define UART1_THR       (*(REG32 (0x01c28400)))
#define UART1_IER       (*(REG32 (0x01c28404)))
#define UART1_IIR       (*(REG32 (0x01c28408)))
#define UART1_FCR       (*(REG32 (0x01c28408)))
#define UART1_LCR       (*(REG32 (0x01c2840C)))
#define UART1_LSR       (*(REG32 (0x01c28414)))
#define UART1_SCR       (*(REG32 (0x01c2841C)))
#define UART1_DLL       (*(REG32 (0x01c28400)))
#define UART1_DLM       (*(REG32 (0x01c28404)))
#define UART1_MCR       (*(REG32 (0x01c28410)))
#define UART1_MSR       (*(REG32 (0x01c28418)))


/*##############################################################################
## I2C
##############################################################################*/

#define I2C_I2CONSET    (*(REG32 (0xE001C000)))
#define I2C_I2STAT      (*(REG32 (0xE001C004)))
#define I2C_I2DAT       (*(REG32 (0xE001C008)))
#define I2C_I2ADR       (*(REG32 (0xE001C00C)))
#define I2C_I2SCLH      (*(REG32 (0xE001C010)))
#define I2C_I2SCLL      (*(REG32 (0xE001C014)))
#define I2C_I2CONCLR    (*(REG32 (0xE001C018)))


/*##############################################################################
## SPI - Serial Peripheral Interface
##############################################################################*/

#define SPI_SPCR        (*(REG32 (0xE0020000)))
#define SPI_SPSR        (*(REG32 (0xE0020004)))
#define SPI_SPDR        (*(REG32 (0xE0020008)))
#define SPI_SPCCR       (*(REG32 (0xE002000C)))
#define SPI_SPTCR       (*(REG32 (0xE0020010)))
#define SPI_SPTSR       (*(REG32 (0xE0020014)))
#define SPI_SPTOR       (*(REG32 (0xE0020018)))
#define SPI_SPINT       (*(REG32 (0xE002001C)))


/*##############################################################################
## Timer 0 and Timer 1
##############################################################################*/

#define TMR_IRQ_EN_REG  (*(REG32 (0x01c20c00)))
#define TMR_IRQ_STA_REG (*(REG32 (0x01c20c04)))

/* ---- Timer 0 -------------------------------------------- */
#define TMR0_CTRL_REG       (*(REG32 (0x01c20c10)))
#define TMR0_INTV_VALUE_REG (*(REG32 (0x01c20c14)))
#define TMR0_CUR_VALUE_REG  (*(REG32 (0x01c20c18)))

/* ---- Timer 1 -------------------------------------------- */
#define TMR1_CTRL_REG       (*(REG32 (0x01c20c10)))
#define TMR1_INTV_VALUE_REG (*(REG32 (0x01c20c14)))
#define TMR1_CUR_VALUE_REG  (*(REG32 (0x01c20c18)))


/*##############################################################################
## PWM
##############################################################################*/

#define PWM_IR          (*(REG32 (0xE0014000)))
#define PWM_TCR         (*(REG32 (0xE0014004)))
#define PWM_TC          (*(REG32 (0xE0014008)))
#define PWM_PR          (*(REG32 (0xE001400C)))
#define PWM_PC          (*(REG32 (0xE0014010)))
#define PWM_MCR         (*(REG32 (0xE0014014)))
#define PWM_MR0         (*(REG32 (0xE0014018)))
#define PWM_MR1         (*(REG32 (0xE001401C)))
#define PWM_MR2         (*(REG32 (0xE0014020)))
#define PWM_MR3         (*(REG32 (0xE0014024)))
#define PWM_MR4         (*(REG32 (0xE0014040)))
#define PWM_MR5         (*(REG32 (0xE0014044)))
#define PWM_MR6         (*(REG32 (0xE0014048)))
#define PWM_EMR         (*(REG32 (0xE001403C)))
#define PWM_PCR         (*(REG32 (0xE001404C)))
#define PWM_LER         (*(REG32 (0xE0014050)))
#define PWM_CCR         (*(REG32 (0xE0014028)))
#define PWM_CR0         (*(REG32 (0xE001402C)))
#define PWM_CR1         (*(REG32 (0xE0014030)))
#define PWM_CR2         (*(REG32 (0xE0014034)))
#define PWM_CR3         (*(REG32 (0xE0014038)))

/*##############################################################################
## RTC
##############################################################################*/

/* ---- RTC: Miscellaneous Register Group ------------------ */
#define RTC_ILR         (*(REG32 (0xE0024000)))
#define RTC_CTC         (*(REG32 (0xE0024004)))
#define RTC_CCR         (*(REG32 (0xE0024008)))  
#define RTC_CIIR        (*(REG32 (0xE002400C)))
#define RTC_AMR         (*(REG32 (0xE0024010)))
#define RTC_CTIME0      (*(REG32 (0xE0024014)))
#define RTC_CTIME1      (*(REG32 (0xE0024018)))
#define RTC_CTIME2      (*(REG32 (0xE002401C)))

/* ---- RTC: Timer Control Group --------------------------- */
#define RTC_SEC         (*(REG32 (0xE0024020)))
#define RTC_MIN         (*(REG32 (0xE0024024)))
#define RTC_HOUR        (*(REG32 (0xE0024028)))
#define RTC_DOM         (*(REG32 (0xE002402C)))
#define RTC_DOW         (*(REG32 (0xE0024030)))
#define RTC_DOY         (*(REG32 (0xE0024034)))
#define RTC_MONTH       (*(REG32 (0xE0024038)))
#define RTC_YEAR        (*(REG32 (0xE002403C)))

/* ---- RTC: Alarm Control Group --------------------------- */
#define RTC_ALSEC       (*(REG32 (0xE0024060)))
#define RTC_ALMIN       (*(REG32 (0xE0024064)))
#define RTC_ALHOUR      (*(REG32 (0xE0024068)))
#define RTC_ALDOM       (*(REG32 (0xE002406C)))
#define RTC_ALDOW       (*(REG32 (0xE0024070)))
#define RTC_ALDOY       (*(REG32 (0xE0024074)))
#define RTC_ALMON       (*(REG32 (0xE0024078)))
#define RTC_ALYEAR      (*(REG32 (0xE002407C)))

/* ---- RTC: Reference Clock Divider Group ----------------- */
#define RTC_PREINT      (*(REG32 (0xE0024080)))
#define RTC_PREFRAC     (*(REG32 (0xE0024084)))


/*##############################################################################
## WD - Watchdog
##############################################################################*/

#define WD_WDMOD        (*(REG32 (0xE0000000)))
#define WD_WDTC         (*(REG32 (0xE0000004)))
#define WD_WDFEED       (*(REG32 (0xE0000008)))
#define WD_WDTV         (*(REG32 (0xE000000C)))


/*##############################################################################
## System Control Block
##############################################################################*/

#define SCB_EXTINT      (*(REG32 (0xE01FC140)))
#define SCB_EXTWAKE     (*(REG32 (0xE01FC144)))
#define SCB_MEMMAP      (*(REG32 (0xE01FC040)))
#define SCB_PLLCON      (*(REG32 (0xE01FC080)))
#define SCB_PLLCFG      (*(REG32 (0xE01FC084)))
#define SCB_PLLSTAT     (*(REG32 (0xE01FC088)))
#define SCB_PLLFEED     (*(REG32 (0xE01FC08C)))
#define SCB_PCON        (*(REG32 (0xE01FC0C0)))
#define SCB_PCONP       (*(REG32 (0xE01FC0C4)))
#define SCB_VPBDIV      (*(REG32 (0xE01FC100)))

/*##############################################################################
## Memory Accelerator Module (MAM)
##############################################################################*/

#define MAM_TIM			(*(REG32 (0xE01FC004)))
#define MAM_CR			(*(REG32 (0xE01FC000)))

#endif /* a83t_h */

