/*
 * FreeRTOS Bananapi M3 port
 * (c) 2020 Dr.-Ing. Ingo D. Rullhusen <d01c@freeshell.de>
 *
 * BPI M3 port is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * BPI M3 port is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FreeRTOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/* Initialization file for Allwinner A83T */

.text
.code 32
.align  0

.extern main_000
.extern main_001
.extern main_002
.extern exit

.extern __bss_beg__
.extern __bss_end__
.extern __stack_end__
.extern __data_beg__
.extern __data_end__
.extern __data+beg_src__

.global start
.global endless_loop

/* Stack Sizes */
.set UND_STACK_SIZE, 0x00100000 //0x00000400
.set ABT_STACK_SIZE, 0x00100000 //0x00000400
.set FIQ_STACK_SIZE, 0x00100000 //0x00000400
.set IRQ_STACK_SIZE, 0x00100000 //0x00000400
.set SVC_STACK_SIZE, 0x00100000 //0x00000400

/* Standard definitions of Mode bits and Interrupt (I & F) flags in PSRs */
.set MODE_USR, 0x10            /* User Mode */
.set MODE_FIQ, 0x11            /* FIQ Mode */
.set MODE_IRQ, 0x12            /* IRQ Mode */
.set MODE_SVC, 0x13            /* Supervisor Mode */
.set MODE_ABT, 0x17            /* Abort Mode */
.set MODE_UND, 0x1B            /* Undefined Mode */
.set MODE_SYS, 0x1F            /* System Mode */

.equ I_BIT, 0x80               /* when I bit is set, IRQ is disabled */
.equ F_BIT, 0x40               /* when F bit is set, FIQ is disabled */


start:
_start:

/* SCTLR.M disable MMU */
   mrc p15, 0, r0, c1, c0, 0
   bic r0, r0, #0x00000001
   mcr p15, 0, r0, c1, c0, 0

/* get CORE ID into r8 */
   mrc p15,0, r8, c0,c0,5
   mov r0,    #0x00000f00
   orr r0,r0, #0x00000003
   and r8,r8, r0

/* one stack per CPU */
init_stack:
   mov r5, #0
   add r5,r5, #UND_STACK_SIZE
   add r5,r5, #ABT_STACK_SIZE
   add r5,r5, #FIQ_STACK_SIZE
   add r5,r5, #IRQ_STACK_SIZE
   add r5,r5, #SVC_STACK_SIZE

   ldr   r0, .LC6

   cmp r8, #0x0
   beq set_stack_0
   cmp r8, #0x00000001
   beq set_stack_1
   cmp r8, #0x00000002
   beq set_stack_2
   cmp r8, #0x00000003
   beq set_stack_3

   mov r4, #0x00000100
   cmp r8, r4
   beq set_stack_4
   add r4,r4, #1
   cmp r8, r4
   beq set_stack_5
   add r4,r4, #1
   cmp r8, r4
   beq set_stack_6
//   add r4,r4, #1
//   cmp r8, r4
//   beq set_stack_7

//set_stack_7:
   sub r0, r0, r5
set_stack_6:
   sub r0, r0, r5
set_stack_5:
   sub r0, r0, r5
set_stack_4:
   sub r0, r0, r5
set_stack_3:
   sub r0, r0, r5
set_stack_2:
   sub r0, r0, r5
set_stack_1:
   sub r0, r0, r5

/* Setup a stack for each mode - note that this only sets up a usable stack
   for system/user, SWI and IRQ modes.   Also each mode is setup with
   interrupts initially disabled. */

set_stack_0:
   msr CPSR_c, #MODE_UND|I_BIT|F_BIT /* Undefined Instruction Mode */
   mov sp, r0
   sub r0, r0, #UND_STACK_SIZE
   msr CPSR_c, #MODE_ABT|I_BIT|F_BIT /* Abort Mode */
   mov sp, r0
   sub r0, r0, #ABT_STACK_SIZE
   msr CPSR_c, #MODE_FIQ|I_BIT|F_BIT /* FIQ Mode */
   mov sp, r0
   sub r0, r0, #FIQ_STACK_SIZE
   msr CPSR_c, #MODE_IRQ|I_BIT|F_BIT /* IRQ Mode */
   mov sp, r0
   sub r0, r0, #IRQ_STACK_SIZE
   msr CPSR_c, #MODE_SVC|I_BIT|F_BIT /* Supervisor Mode */
   mov sp, r0
   sub r0, r0, #SVC_STACK_SIZE
   msr CPSR_c, #MODE_SYS|I_BIT|F_BIT /* System Mode */
   mov sp, r0


/* Set VBAR in system mode to register value. */
/* SCTLR */
   mrc p15, 0, r1, c1, c0, 0
   bic r1, r1, #0x00002000
   bic r1, r1, #0x01000000
   mcr p15, 0, r1, c1, c0, 0
/* VBAR */
   mrc p15, 0, r1, c12, c0, 0
   ldr r1, =__vba__
   mcr p15, 0, r1, c12, c0, 0


/* We want to start in supervisor mode.  Operation will switch to system
   mode when the first task starts. */
   msr   CPSR_c, #MODE_SVC|I_BIT|F_BIT


/* Init done by CPU0 only. */
   cmp r8, #0x0
   bne .end_cpu0_init


/* Clear BSS. */
   mov r1, #0 /* Fill value */
   mov fp, r1 /* Null frame pointer */
   mov r7, r1 /* Null frame pointer for Thumb */

   ldr r1, .LC1 /* Start of memory block */
   ldr r3, .LC2 /* End of memory block */
   subs r3, r3, r1 /* Length of block */
   beq .end_clear_loop
   mov r2, #0

.clear_loop:
   strb r2, [r1], #1
   subs r3, r3, #1
   bgt .clear_loop

.end_clear_loop:


/* Initialise data. */
   ldr r1, .LC3 /* Start of memory block */
   ldr r2, .LC4 /* End of memory block */
   subs r3, r2, r1 /* In place */
   beq .end_set_loop

   ldr r3, .LC5
   subs r3, r3, r1 /* Length of block */
   beq .end_set_loop

.set_loop:
   ldrb r4, [r2], #1
   strb r4, [r1], #1
   subs r3, r3, #1
   bgt .set_loop

.end_set_loop:

/* Set soft start address. */
   ldr r1, =0x01f01c00 /* R_CPUCFG */
   add r1, #0x01a4 /* R_CPUCFG_CPU_SOFT_ENTRY_REG */
   ldr r0, .LC0
   str r0, [r1]

/* init spinlock */
   ldr r1, =0x01c20000 // CCU
   add r1, #0x0064 // BUS_CLOCK_GATING_REG1
   ldr r0, [r1]
   orr r0, r0, #0x00400000
   str r0, [r1]

   ldr r1, =0x01c20000 // CCU 
   add r1, #0x02c4 // BUS_SOFTWARE_RESET_REG1
   ldr r0, [r1]
   orr r0, r0, #0x00400000
   str r0, [r1]

.end_cpu0_init:


start_main:
   mov r0, #0 /* no arguments  */
   mov r1, #0 /* no argv either */

   cmp r8, #0x00000000
   bleq main_000
   cmp r8, #0x00000001
   bleq main_001
   cmp r8, #0x00000002
   bleq main_002
   cmp r8, #0x00000003
   bleq main_003

   mov r4, #0x00000100
   cmp r8, r4
   bleq main_100
   add r4,r4, #1
   cmp r8, r4
   bleq main_101
   add r4,r4, #1
   cmp r8, r4
   bleq main_102
   add r4,r4, #1
   cmp r8, r4
   bleq main_103

   bl main_000 /* never reached */

endless_loop:
   b endless_loop


.align 0

.LC0: .word _start
.LC1: .word __bss_beg__
.LC2: .word __bss_end__
.LC3: .word __data_beg__
.LC4: .word __data_beg_src__
.LC5: .word __data_end__
.LC6: .word __stack_end__


/* Setup vector table.  Note that undf, pabt, dabt, fiq just execute
   a null loop. */

.section .startup,"ax"
   .code 32
   .align 0

   ldr pc, _rst  /* reset - _start */
   ldr pc, _undf /* undefined - _undf */
   ldr pc, _swi  /* SWI - _swi */
   ldr pc, _pabt /* program abort - _pabt */
   ldr pc, _dabt /* data abort - _dabt */
   nop           /* reserved */
   ldr pc, _irq  /* IRQ - read the VIC */
   ldr pc, _fiq  /* FIQ - _fiq */
   ldr pc, _fel  /* FEL setup */

_rst:  .word _start              /* start */
_undf: .word __undf              /* undefined */
_swi:  .word vPortYieldProcessor /* SWI */
_pabt: .word __pabt              /* program abort */
_dabt: .word __dabt              /* data abort */
_irq:  .word vTickISR            /* IRQ */
_fiq:  .word __fiq               /* FIQ */
_fel:  .word __fel               /* FEL */

__undf: b . /* undefined */
__pabt: b . /* program abort */
__dabt: b . /* data abort */
__fiq:  b . /* FIQ */
__fel:  b . /* FEL */

