/*
 * FreeRTOS Bananapi M3 port
 * Main for Core 000 (Boot/Config Core)
 * (c) 2020 Dr.-Ing. Ingo D. Rullhusen <d01c@freeshell.de>
 *
 * BPI M3 port is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * BPI M3 port is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FreeRTOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "rtos-syscalls.h"

#include "FreeRTOS.h"
#include "task.h"

#include "serial.h"
#include "printk.h"
#include "partest.h"

#include "blink_task.h"
#include "cmd_task.h"
#include "mon_task.h"

/*-----------------------------------------------------------*/


extern BlinkTaskParameter xGreen;
extern BlinkTaskParameter xBlue;


CmdTaskParameter xCmd;
extern MonTaskParameter xMon[8];

extern uint8_t cores_ready;
extern volatile unsigned int *spinlock_cores_ready;


static void prvSetupHardware( void )
{
// UART0
   xSerialPortInitMinimal( 115200, 1024 );

   GICD_CTLR |= 0x00000040; // 1: non-secure access 
   GICD_CTLR |= 0x00000003; // 2: enable

// Initialise LED outputs.
   vParTestInitialise();


//
// PB3 (in/pull-up) EINT active low
   PB_DATA_REG = 0xffffffff; // high
   PB_PULL0_REG = 0x00000040; // pull-up
   PB_EINT_CFG0_REG = 0x00003000; // active low
   for ( int i=0; i<1000; ++i ){} // prevent glitch
   PB_EINT_STATUS_REG = 0x00000008; // clear
   PB_CFG0_REG = 0x77776777; // PB_EINT3
   PB_EINT_CTL_REG = 0x00000008; // enable eint


//
// GIC PB_EINT = interrupt 47 ; n = m DIV 32
   GICD_IGROUPR(1) |= 0x00000000; // all group 0
   GICD_ISENABLER(1) |= 0x00008000; // enable interrupt 47, bit 15
   GICD_IPRIORITYR( (47/4) ) |= 0xf0000000;
   GICD_ITARGETSR( (47/4) ) |= 0x10000000; // core 100


   configSETUP_TICK_INTERRUPT();


   cores_ready = 0x00;
   spinlock_cores_ready = &SPINLOCK_LOCK_REG(30);

//
// Cluster0 / CPU1...3 Power

   C0CPUX_PWROFF_GATING_REG |= 0xe;
   C0_CPU1_PWR_SWITCH_CTRL = 0xff;
   C0_CPU2_PWR_SWITCH_CTRL = 0xff;
   C0_CPU3_PWR_SWITCH_CTRL = 0xff;
   for ( int i=0; i<10000; ++i ){}

   C0_CPU_PO_RST_CTRL &= ~0x0000000e;

   R_C0_PO_RST_CTRL &= ~0x0000000e;
   for ( int i=0; i<10000; ++i ){}

   C0_CTRL_REG0 &= ~0x12; // reset L2 L1
   
   C0_RST_CTRL &= ~0x0022000e; // DBG ETM CORE

   for ( int i=0; i<10000; ++i ){}
   C0_CPU1_PWR_SWITCH_CTRL = 0xfe;
   C0_CPU2_PWR_SWITCH_CTRL = 0xfe;
   C0_CPU3_PWR_SWITCH_CTRL = 0xfe;
   for ( int i=0; i<10000; ++i ){}
   C0_CPU1_PWR_SWITCH_CTRL = 0xf8;
   C0_CPU2_PWR_SWITCH_CTRL = 0xf8;
   C0_CPU3_PWR_SWITCH_CTRL = 0xf8;
   for ( int i=0; i<10000; ++i ){}
   C0_CPU1_PWR_SWITCH_CTRL = 0xf0;
   C0_CPU2_PWR_SWITCH_CTRL = 0xf0;
   C0_CPU3_PWR_SWITCH_CTRL = 0xf0;
   for ( int i=0; i<10000; ++i ){}
   C0_CPU1_PWR_SWITCH_CTRL = 0x00;
   C0_CPU2_PWR_SWITCH_CTRL = 0x00;
   C0_CPU3_PWR_SWITCH_CTRL = 0x00;
   for ( int i=0; i<10000; ++i ){}

   C0CPUX_PWROFF_GATING_REG &= ~0xe;
   for ( int i=0; i<20000; ++i ){}


   C0_CPU_PO_RST_CTRL |= 0xe;
   
   R_C0_PO_RST_CTRL |= 0xe;
   for ( int i=0; i<10000; ++i ){}

   C0_RST_CTRL |= 0x0022000e; // DBG ETM CORE

   for ( int i=0; i<20000; ++i ){}


//
// Cluster1 / CPU0...3 Power

   C1CPUX_PWROFF_GATING_REG |= 0x1f; // Cluster and CPU0...3
   C1_CPU0_PWR_SWITCH_CTRL = 0xff;
   C1_CPU1_PWR_SWITCH_CTRL = 0xff;
   C1_CPU2_PWR_SWITCH_CTRL = 0xff;
   C1_CPU3_PWR_SWITCH_CTRL = 0xff;
   for ( int i=0; i<10000; ++i ){}

   C1_CPU_PO_RST_CTRL &= ~0x0000000f;

   R_C1_PO_RST_CTRL &= ~0x0000000f;
   for ( int i=0; i<10000; ++i ){}

   C1_CTRL_REG0 &= ~0x12; // reset L2 L1

   C1_CTRL_REG1 |= 0x1; // ACINACTM
   
   C1_RST_CTRL &= ~0x0022000f; // DBG ETM CORE

   for ( int i=0; i<10000; ++i ){}
   C1_CPU0_PWR_SWITCH_CTRL = 0xfe;
   C1_CPU1_PWR_SWITCH_CTRL = 0xfe;
   C1_CPU2_PWR_SWITCH_CTRL = 0xfe;
   C1_CPU3_PWR_SWITCH_CTRL = 0xfe;
   for ( int i=0; i<10000; ++i ){}
   C1_CPU0_PWR_SWITCH_CTRL = 0xf8;
   C1_CPU1_PWR_SWITCH_CTRL = 0xf8;
   C1_CPU2_PWR_SWITCH_CTRL = 0xf8;
   C1_CPU3_PWR_SWITCH_CTRL = 0xf8;
   for ( int i=0; i<10000; ++i ){}
   C1_CPU0_PWR_SWITCH_CTRL = 0xf0;
   C1_CPU1_PWR_SWITCH_CTRL = 0xf0;
   C1_CPU2_PWR_SWITCH_CTRL = 0xf0;
   C1_CPU3_PWR_SWITCH_CTRL = 0xf0;
   for ( int i=0; i<10000; ++i ){}
   C1_CPU0_PWR_SWITCH_CTRL = 0x00;
   C1_CPU1_PWR_SWITCH_CTRL = 0x00;
   C1_CPU2_PWR_SWITCH_CTRL = 0x00;
   C1_CPU3_PWR_SWITCH_CTRL = 0x00;
   for ( int i=0; i<10000; ++i ){}

   C1CPUX_PWROFF_GATING_REG &= ~0x1f;
   for ( int i=0; i<20000; ++i ){}

   C1_CPU_PO_RST_CTRL |= 0xf;
   
   R_C1_PO_RST_CTRL |= 0xf;
   for ( int i=0; i<10000; ++i ){}

   C1_RST_CTRL |= 0x0022000f; // DBG ETM CORE

   C1_CTRL_REG1 &= ~0x1; // ACINACTM

   for ( int i=0; i<20000; ++i ){}


}

void vConfigureTickInterrupt()
{
   uint32_t ulCompareMatch;

/* A 1ms tick does not require the use of the timer prescale.  This is
   defaulted to zero but can be used if necessary. */

/* Calculate the match value required for our wanted tick rate. */
   ulCompareMatch = (24000000 / configTICK_RATE_HZ);

   TMR0_INTV_VALUE_REG = ulCompareMatch;

/* Start the timer - interrupts are disabled when this function is called
   so it is okay to do this here. */
   TMR0_CTRL_REG = 0x000000007; // continuous, /1, 24MHz, reload, start

   TMR_IRQ_EN_REG |= 0x00000001;
   TMR_IRQ_STA_REG |= 0x00000001;

//
// GIC timer0 = interrupt 50 ; n = m DIV 32

   GICD_IGROUPR(1) |= 0x00000000; // all group 0
   GICD_ISENABLER(1) |= 0x00040000; // enable interrupt 50, bit 18
   GICD_IPRIORITYR( (50/4) ) |= 0x00000000;
   GICD_ITARGETSR( (50/4) ) |= 0x00010000; // core 0 only, other via SGI

//
// GIC SGI 0...15
   GICD_IGROUPR(0) |= 0x00000000; // all group 0
   GICD_ISENABLER(0) |= 0x0000ffff; // enable interrupt 0...15
   GICD_IPRIORITYR( (0/4) ) |= 0x00000000;

}


void vApplicationStackOverflowHook( TaskHandle_t *pxTask, signed char *pcTaskName );
void vApplicationStackOverflowHook( TaskHandle_t *pxTask, signed char *pcTaskName )
{
   (void) pxTask;
   printk( "STACKOVERFLOW_000: %s\n", pcTaskName );
}

void vApplicationMallocFailedHook( void );
void vApplicationMallocFailedHook()
{
   printk("MALLOC_000 failed!\n");
}


void vApplicationIdleHook( void );
void vApplicationIdleHook( void )
{ __asm( "wfi" ); }


int main_000( void );
int main_000( void )
{
// Setup the hardware for use with the Banana Pi M3 demo board.

   uint32_t MPIDR; /* 0x00000f00: ClusterID, 0x00000003: CpuID */
   __asm( "mrc p15,0,%0,c0,c0,5" : "=r" ( MPIDR ) : );
   uint16_t CORE_ID = 0x00000f03 & MPIDR;
   if ( CORE_ID != 0x000 )
   {
      printk( "000: Wrong Core! Stopped.\n" );
      while(1){ __asm( "wfi" ); }
      return(0);
   }

   prvSetupHardware(); // start cores 1...7

   GICC_CTLR = 0x000000001; // group 0 and 1
   GICC_PMR = 0xffffffff;
   GICC_BPR = 0x00000000;


   xTaskCreate( CmdTaskFunction, "Cmd", configMINIMAL_STACK_SIZE, (void*)&xCmd, 1, &xCmd.xHandle );

   xTaskCreate( MonTaskFunction, "Mon_000", configMINIMAL_STACK_SIZE, (void*)&(xMon[0]), 1, &(xMon[0]).xHandle );


   while ( *spinlock_cores_ready ) {}
   printk( "CORE 000 ready.\n" );
   cores_ready |= 0x01;
   *spinlock_cores_ready = 0;
   while ( cores_ready != 0xff ) {}


// Now all the tasks and cores have been started - start the scheduler.

//	NOTE : Tasks run in system mode and the scheduler runs in Supervisor mode.
//	The processor MUST be in supervisor mode when vTaskStartScheduler is
//	called.  The demo applications included in the FreeRTOS.org download switch
//	to supervisor mode prior to main being called.  If you are not using one of
//	these demo application projects then ensure Supervisor mode is used here.
   vTaskStartScheduler();

// Should never reach here!
   return(0);
}


void vUART_ISR_Handler( void ) __attribute__ ((noinline));

void vISR_000( void ) __attribute__((naked));
void vISR_000( void )
{
   portSAVE_CONTEXT();

   uint32_t MPCoreInterruptID = GICC_IAR;
   uint16_t IID = MPCoreInterruptID & 0x000003ff;

   switch ( IID )
   {
      case 32: // UART
      {
         vUART_ISR_Handler();
         break;
      }

      case 50: // timer
      {
         if ( xTaskIncrementTick() != pdFALSE )
         {
            // Find the highest priority task that is ready to run.
            vTaskSwitchContext();
         }

         TMR_IRQ_STA_REG |= 0x00000001;

         GICD_SGIR = (0xfe<<16); // tick to all other

         break;
      }

      default:
      {
         uint32_t SP;
         __asm( "mov %0, sp" : "=r" ( SP ) : );

         printk("%i:INTERRUPT_000: %u sp:0x%08x \n",  xTaskGetTickCountFromISR(), IID, SP);

         break;
      }
   }

   GICC_EOIR = MPCoreInterruptID;
   portRESTORE_CONTEXT();

}

void vPLP_000( void ) __attribute__((naked));
void vPLP_000( void )
{
/* Within an IRQ ISR the link register has an offset from the true return
   address, but an SWI ISR does not.  Add the offset manually so the same
   ISR return code can be used in both cases. */
   __asm volatile ( "ADD LR, LR, #4" );

/* Perform the context switch.  First save the context of the current task. */
   portSAVE_CONTEXT();

/* Find the highest priority task that is ready to run. */
   __asm volatile ( "bl vTaskSwitchContext" );

/* Restore the context of the new task. */
   portRESTORE_CONTEXT();
}

