/*
 * FreeRTOS Bananapi M3 port
 * Main for Core 002
 * (c) 2020 Dr.-Ing. Ingo D. Rullhusen <d01c@freeshell.de>
 *
 * BPI M3 port is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * BPI M3 port is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FreeRTOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "rtos-syscalls.h"

#include "FreeRTOS.h"
#include "task.h"

#include "printk.h"

#include "mon_task.h"

/*-----------------------------------------------------------*/


extern MonTaskParameter xMon[8];

extern uint8_t cores_ready;
extern volatile unsigned int *spinlock_cores_ready;


void vApplicationStackOverflowHook( TaskHandle_t *pxTask, signed char *pcTaskName );
void vApplicationStackOverflowHook( TaskHandle_t *pxTask, signed char *pcTaskName )
{
   (void) pxTask;
   printk( "STACKOVERFLOW_002: %s\n", pcTaskName );
}

void vApplicationMallocFailedHook( void );
void vApplicationMallocFailedHook()
{
   printk("MALLOC_002 failed!\n");
}


void vApplicationIdleHook( void );
void vApplicationIdleHook( void )
{ __asm( "wfi" ); }


int main_002( void );
int main_002( void )
{
   uint32_t MPIDR; /* 0x00000f00: ClusterID, 0x00000003: CpuID */
   __asm( "mrc p15,0,%0,c0,c0,5" : "=r" ( MPIDR ) : );
   uint16_t CORE_ID = 0x00000f03 & MPIDR;
   if ( CORE_ID != 0x002 )
   {
      printk( "002: Wrong Core! Stopped.\n" );
      while(1){ __asm( "wfi" ); }
      return(0);
   }

   GICC_CTLR = 0x000000001; // group 0 and 1
   GICC_PMR = 0xffffffff;
   GICC_BPR = 0x00000000;


   xTaskCreate( MonTaskFunction, "Mon_002", configMINIMAL_STACK_SIZE, (void*)&xMon[2], 1, &(xMon[2]).xHandle );


   while ( *spinlock_cores_ready ) {} /* single core begin */
   printk("CORE 002 ready.\n");
   cores_ready |= 0x04;
   *spinlock_cores_ready = 0; /* single core end */
   while ( cores_ready != 0xff ) {}


// Now all the tasks and cores have been started - start the scheduler.

//	NOTE : Tasks run in system mode and the scheduler runs in Supervisor mode.
//	The processor MUST be in supervisor mode when vTaskStartScheduler is
//	called.  The demo applications included in the FreeRTOS.org download switch
//	to supervisor mode prior to main being called.  If you are not using one of
//	these demo application projects then ensure Supervisor mode is used here.
   vTaskStartScheduler();

// Should never reach here!
   return(0);
}


void vISR_002( void ) __attribute__((naked));
void vISR_002( void )
{
   portSAVE_CONTEXT();

   uint32_t MPCoreInterruptID = GICC_IAR;
   uint16_t IID = MPCoreInterruptID & 0x000003ff;

   switch ( IID )
   {
      case 0: // timer by SGI from core 0
      {
         if ( xTaskIncrementTick() != pdFALSE )
         {
            // Find the highest priority task that is ready to run.
            vTaskSwitchContext();
         }
         break;
      }

      default:
      {
         uint32_t SP;
         __asm( "mov %0, sp" : "=r" ( SP ) : );

         printk("%i:INTERRUPT_002: %u sp:0x%08x \n",  xTaskGetTickCountFromISR(), IID, SP);

         break;
      }
   }

   GICC_EOIR = MPCoreInterruptID;
   portRESTORE_CONTEXT();
}

void vPLP_002( void ) __attribute__((naked));
void vPLP_002( void )
{
/* Within an IRQ ISR the link register has an offset from the true return
   address, but an SWI ISR does not.  Add the offset manually so the same
   ISR return code can be used in both cases. */
   __asm volatile ( "ADD LR, LR, #4" );

/* Perform the context switch.  First save the context of the current task. */
   portSAVE_CONTEXT();

/* Find the highest priority task that is ready to run. */
   __asm volatile ( "bl vTaskSwitchContext" );

/* Restore the context of the new task. */
   portRESTORE_CONTEXT();
}

