/*
 * FreeRTOS Bananapi M3 port
 * (c) 2020 Dr.-Ing. Ingo D. Rullhusen <d01c@freeshell.de>
 *
 * BPI M3 port is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * BPI M3 port is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FreeRTOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <stdio.h>

#include "mon_task.h"

#include "printk.h"


static void mon_status( char *buffer, int size )
{
   uint32_t MPIDR; /* 0x00000f00: ClusterID, 0x00000003: CpuID */
   __asm( "mrc p15,0,%0,c0,c0,5" : "=r" ( MPIDR ) : );
   uint16_t CORE_ID = 0x00000f03 & MPIDR;

   int n;

   n = snprintf( buffer, size, "Core: 0x%03x Ticks: %i\n", CORE_ID, xTaskGetTickCount() );
   buffer += n; size -= n;
   n = snprintf( buffer, size, "\n" );
   buffer += n; size -= n;

   TaskStatus_t *pxTaskStatusArray;
   volatile UBaseType_t uxArraySize, x;

// Take a snapshot of the number of tasks in case it changes while this
// function is executing.
   uxArraySize = uxTaskGetNumberOfTasks();

// Allocate a TaskStatus_t structure for each task.  An array could be
// allocated statically at compile time.
   pxTaskStatusArray = pvPortMalloc( uxArraySize * sizeof( TaskStatus_t ) );

   n = snprintf( buffer, size, "TaskName        \tTask#\tState\tPri\tBPri\tStkHWM\n" );
   buffer += n; size -= n;
   if ( pxTaskStatusArray != NULL )
   {
// Generate raw status information about each task.
      uxArraySize = uxTaskGetSystemState( pxTaskStatusArray, uxArraySize, NULL );

// For each populated position in the pxTaskStatusArray array,
// format the raw data as human readable ASCII data
      for( x = 0; x < uxArraySize; x++ )
      {
         n = snprintf( buffer, size, "%16s\t%lu\t%u\t%lu\t%lu\t%u\n",
                 pxTaskStatusArray[ x ].pcTaskName,
                 pxTaskStatusArray[ x ].xTaskNumber,
                 pxTaskStatusArray[ x ].eCurrentState,
                 pxTaskStatusArray[ x ].uxCurrentPriority,
                 pxTaskStatusArray[ x ].uxBasePriority,
                 pxTaskStatusArray[ x ].usStackHighWaterMark );
         buffer += n; size -= n;
      }
   }
// The array is no longer needed, free the memory it consumes.
   vPortFree( pxTaskStatusArray );

}


portTASK_FUNCTION( MonTaskFunction, pvParameters )
{
   MonTaskParameter *param = (MonTaskParameter*)pvParameters;

   param->ctrl = 0;
   param->lastStart = xTaskGetTickCount();

   for( ; ; )
   {
      param->lastStart = xTaskGetTickCount(); /* wakeable */

//      uint32_t MPIDR; /* 0x00000f00: ClusterID, 0x00000003: CpuID */
//      __asm( "mrc p15,0,%0,c0,c0,5" : "=r" ( MPIDR ) : );
//      uint16_t CORE_ID = 0x00000f03 & MPIDR;

      while ( param->ctrl != 1 ) { taskYIELD(); }
      
      mon_status( param->answer, MonResponseBufferSize );

      param->ctrl = 2;

   }

   vTaskDelete( NULL );
}

