/*
FUNCTION
<<__retarget_lock_init>>, <<__retarget_lock_init_recursive>>, <<__retarget_lock_close>>, <<__retarget_lock_close_recursive>>, <<__retarget_lock_acquire>>, <<__retarget_lock_acquire_recursive>>, <<__retarget_lock_try_acquire>>, <<__retarget_lock_try_acquire_recursive>>, <<__retarget_lock_release>>, <<__retarget_lock_release_recursive>>---locking routines

INDEX
	__lock___sinit_recursive_mutex
INDEX
	__lock___sfp_recursive_mutex
INDEX
	__lock___atexit_recursive_mutex
INDEX
	__lock___at_quick_exit_mutex
INDEX
	__lock___malloc_recursive_mutex
INDEX
	__lock___env_recursive_mutex
INDEX
	__lock___tz_mutex
INDEX
	__lock___dd_hash_mutex
INDEX
	__lock___arc4random_mutex

INDEX
	__retarget_lock_init
INDEX
	__retarget_lock_init_recursive
INDEX
	__retarget_lock_close
INDEX
	__retarget_lock_close_recursive
INDEX
	__retarget_lock_acquire
INDEX
	__retarget_lock_acquire_recursive
INDEX
	__retarget_lock_try_acquire
INDEX
	__retarget_lock_try_acquire_recursive
INDEX
	__retarget_lock_release
INDEX
	__retarget_lock_release_recursive

SYNOPSIS
	#include <lock.h>
	struct __lock __lock___sinit_recursive_mutex;
	struct __lock __lock___sfp_recursive_mutex;
	struct __lock __lock___atexit_recursive_mutex;
	struct __lock __lock___at_quick_exit_mutex;
	struct __lock __lock___malloc_recursive_mutex;
	struct __lock __lock___env_recursive_mutex;
	struct __lock __lock___tz_mutex;
	struct __lock __lock___dd_hash_mutex;
	struct __lock __lock___arc4random_mutex;

	void __retarget_lock_init (_LOCK_T * <[lock_ptr]>);
	void __retarget_lock_init_recursive (_LOCK_T * <[lock_ptr]>);
	void __retarget_lock_close (_LOCK_T <[lock]>);
	void __retarget_lock_close_recursive (_LOCK_T <[lock]>);
	void __retarget_lock_acquire (_LOCK_T <[lock]>);
	void __retarget_lock_acquire_recursive (_LOCK_T <[lock]>);
	int __retarget_lock_try_acquire (_LOCK_T <[lock]>);
	int __retarget_lock_try_acquire_recursive (_LOCK_T <[lock]>);
	void __retarget_lock_release (_LOCK_T <[lock]>);
	void __retarget_lock_release_recursive (_LOCK_T <[lock]>);

DESCRIPTION
Newlib was configured to allow the target platform to provide the locking
routines and static locks at link time.  As such, a dummy default
implementation of these routines and static locks is provided for
single-threaded application to link successfully out of the box on bare-metal
systems.

For multi-threaded applications the target platform is required to provide
an implementation for @strong{all} these routines and static locks.  If some
routines or static locks are missing, the link will fail with doubly defined
symbols.

PORTABILITY
These locking routines and static lock are newlib-specific.  Supporting OS
subroutines are required for linking multi-threaded applications.
*/

/* Allwinner A83T lock routines */


#include <sys/lock.h>

#include "FreeRTOS.h"
#include "printk.h"


struct __lock {
   unsigned int type;
   unsigned char used;
};

struct __lock __lock___sinit_recursive_mutex  = { .type = 1, .used = 0 };
struct __lock __lock___sfp_recursive_mutex    = { .type = 2, .used = 0 };
struct __lock __lock___atexit_recursive_mutex = { .type = 3, .used = 0 };
struct __lock __lock___at_quick_exit_mutex    = { .type = 4, .used = 0 };
struct __lock __lock___malloc_recursive_mutex = { .type = 5, .used = 0 };
struct __lock __lock___env_recursive_mutex    = { .type = 6, .used = 0 };
struct __lock __lock___tz_mutex               = { .type = 7, .used = 0 };
struct __lock __lock___dd_hash_mutex          = { .type = 8, .used = 0 };
struct __lock __lock___arc4random_mutex       = { .type = 9, .used = 0 };

static unsigned int next_mutex_type = 10;

static uint32_t mask_old;
static unsigned int masked_by = 0;

void
__retarget_lock_init (_LOCK_T *lock)
{
   printk("__retarget_lock_init %i (not implemented)\n",(*lock)->type);
}

void
__retarget_lock_init_recursive(_LOCK_T *lock)
{
   (*lock)->type = next_mutex_type++;
   (*lock)->used = 0;
}

void
__retarget_lock_close(_LOCK_T lock)
{
   (void) lock;
   printk("__retarget_lock_close (not implemented)\n");
}

void
__retarget_lock_close_recursive(_LOCK_T lock)
{
   if ( lock->type == next_mutex_type-1 ) { --next_mutex_type; }
}

void
__retarget_lock_acquire (_LOCK_T lock)
{
   (void) lock;
   printk("__retarget_lock_acquire (not implemented)\n");
}

void
__retarget_lock_acquire_recursive (_LOCK_T lock)
{

   if ( lock->type == 5 )
   {
      volatile unsigned int *spinlock = &SPINLOCK_LOCK_REG(31);
      while ( *spinlock ) {} /* single core begin */
   }

   ++(lock->used);

   if ( masked_by == 0 )
   {
      __asm( "mrs %0, CPSR\n"
             "orr r0, %0, #0x000000c0\n"
             "msr CPSR_c, r0\n"
             : "=r" (mask_old) : : "r0" );
      masked_by = lock->type;
   }

}

int
__retarget_lock_try_acquire(_LOCK_T lock)
{
   (void) lock;
   printk("__retarget_lock_try_acquire (not implemented)\n");
   return 1;
}

int
__retarget_lock_try_acquire_recursive(_LOCK_T lock)
{

   if ( lock->type == 5 )
   {
      volatile unsigned int *spinlock = &SPINLOCK_LOCK_REG(31);
      if ( *spinlock ) { return(0); }
   }

   ++(lock->used);

   if ( masked_by == 0 )
   {
      __asm( "mrs %0, CPSR\n"
             "orr r0, %0, #0x000000c0\n"
             "msr CPSR_c, r0\n"
             : "=r" (mask_old) : : "r0" );
      masked_by = lock->type;
   }

   return(1);
}

void
__retarget_lock_release (_LOCK_T lock)
{
   (void) lock;
   printk("__retarget_lock_release %i (not implemented)\n",lock->type);
}

void
__retarget_lock_release_recursive (_LOCK_T lock)
{

   --(lock->used);

   if ( lock->used < 1 )
   {
      lock->used = 0;
      if ( masked_by == lock->type )
      {
         masked_by = 0;
         __asm( "msr CPSR_c, %0\n" : : "r" (mask_old) : );
      }
   }

   if ( lock->type == 5 )
   {
      volatile unsigned int *spinlock = &SPINLOCK_LOCK_REG(31);
      if ( lock->used < 1 )
      {
         *spinlock = 0;
      }
   }

}

