/*
 * FreeRTOS Bananapi M3 port
 * Switch Global ISR to Core Context ISR
 * (c) 2020 Dr.-Ing. Ingo D. Rullhusen <d01c@freeshell.de>
 *
 * BPI M3 port is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * BPI M3 port is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FreeRTOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


/* Scheduler includes. */
#include "FreeRTOS.h"


void vPortYieldProcessor( void ) __attribute__((interrupt("SWI"), naked));

void vPortYieldProcessor( void )
{

   /* 0x00000f00: ClusterID, 0x00000003: CpuID => 0...7 */
   __asm(
          "push { r7, r8 }\n"

          "mrc   p15,0,r8,c0,c0,5\n"
          "tst   r8,    #0x00000f00\n"
          "addne r8,r8, #4\n"
          "and   r8,r8, #0x00000007\n"

          "adr   r7, plp_jmp_table\n"
          "add   pc,r7, r8, lsl #3\n"

          "plp_jmp_table:\n"

          "pop { r7, r8 }\n"
          "b vPLP_000\n"
          "pop { r7, r8 }\n"
          "b vPLP_001\n"
          "pop { r7, r8 }\n"
          "b vPLP_002\n"
          "pop { r7, r8 }\n"
          "b vPLP_003\n"
          "pop { r7, r8 }\n"
          "b vPLP_100\n"
          "pop { r7, r8 }\n"
          "b vPLP_101\n"
          "pop { r7, r8 }\n"
          "b vPLP_102\n"
          "pop { r7, r8 }\n"
          "b vPLP_103\n"
        );

/* not reached */
}


void vTickISR( void ) __attribute__((naked));

void vTickISR( void )
{

   /* 0x00000f00: ClusterID, 0x00000003: CpuID => 0...7 */
   __asm(
          "push { r7, r8 }\n"

          "mrc   p15,0,r8,c0,c0,5\n"
          "tst   r8,    #0x00000f00\n"
          "addne r8,r8, #4\n"
          "and   r8,r8, #0x00000007\n"

          "adr   r7, isr_jmp_table\n"
          "add   pc,r7, r8, lsl #3\n"

          "isr_jmp_table:\n"

          "pop { r7, r8 }\n"
          "b vISR_000\n"
          "pop { r7, r8 }\n"
          "b vISR_001\n"
          "pop { r7, r8 }\n"
          "b vISR_002\n"
          "pop { r7, r8 }\n"
          "b vISR_003\n"
          "pop { r7, r8 }\n"
          "b vISR_100\n"
          "pop { r7, r8 }\n"
          "b vISR_101\n"
          "pop { r7, r8 }\n"
          "b vISR_102\n"
          "pop { r7, r8 }\n"
          "b vISR_103\n"
        );

/* not reached */
}

