/*
 * FreeRTOS Bananapi M3 port
 * syscalls for newlib
 *
 * (c) 2020 Dr.-Ing. Ingo D. Rullhusen <d01c@freeshell.de>
 *
 * BPI M3 port is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * BPI M3 port is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FreeRTOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "FreeRTOS.h"
#include "task.h"

#include "serial.h"

#include "rtos-syscalls.h"


#include <errno.h>
#undef errno
extern int errno;


char *__env[1] = { 0 };
char **environ = __env;


void _exit( int status )
{
   (void) status;
   for ( ; ; ){}
}


void *_sbrk ( ptrdiff_t delta )
{
   void *retval = (void *) -1;
   extern char _end; /* Defined by the linker */
   static char *heap_end = &_end;
 
   if ( (heap_end - &_end) + delta > (ptrdiff_t)configTOTAL_HEAP_SIZE )
   {
      errno = ENOMEM;
   }
   else
   {
      retval = (void *) heap_end;
      heap_end += delta;
   }

   return( retval );
}


int _read( int file, char *data, int len )
{
   int bytes_read = 0;

   uint32_t MPIDR=0; /* 0x00000f00: ClusterID, 0x00000003: CpuID */
   __asm( "mrc p15,0,%0,c0,c0,5" : "=r" ( MPIDR ) : );

   if ( ( 0x00000f03 & MPIDR ) || ( file != STDIN_FILENO ) ) /* core 0 only */
   {
      errno = EBADF;
      return(-1);
   }

   signed char cRxedChar;
   while ( bytes_read < len )
   {
      while ( xSerialGetChar( 0, &cRxedChar, 0 ) != pdTRUE ) {}

      if ( cRxedChar == '\r' ) { cRxedChar = '\n'; }
      data[ bytes_read++ ] = cRxedChar;
      if ( cRxedChar == '\n' ) break;

   }

   return( bytes_read );
}


int _write( int file, char *data, int len )
{
   int bytes_written;

   uint32_t MPIDR=0; /* 0x00000f00: ClusterID, 0x00000003: CpuID */
   __asm( "mrc p15,0,%0,c0,c0,5" : "=r" ( MPIDR ) : );

   if ( ( 0x00000f03 & MPIDR ) ||
        ( (file != STDOUT_FILENO) && (file != STDERR_FILENO) ) ) /* core 0 only */
   {
      errno = EBADF;
      return(-1);
   }

   for ( bytes_written = 0; bytes_written < len; ++bytes_written )
   {
      signed char cTxChar = (signed char)*(data++);

      if ( xSerialPutChar( 0, cTxChar, 0 ) != pdPASS ) break;
      if ( cTxChar == '\n' ) { if ( xSerialPutChar( 0, '\r', 0 ) != pdPASS ) break; }
   }

   return( bytes_written );
}


int _kill( int pid, int sig )
{
   (void) pid;
   (void) sig;
   errno = EINVAL;
   return(-1);
}

int _close( int file )
{
   (void) file;
   errno = EBADF;
   return(-1);
}

int _lseek( int file, int ptr, int dir )
{
   (void) file;
   (void) ptr;
   (void) dir;
   errno = ESPIPE;
   return(-1);
}

int _getpid(void) { return(1); }

int _fstat( int file, struct stat *st )
{
   (void) file;
   st->st_mode = S_IFCHR;
   return(0);
}

int _isatty( int file )
{
   if ( (file == STDOUT_FILENO) ||
        (file == STDIN_FILENO ) ||
        (file == STDERR_FILENO)   )
   {
      return(1);
   }

   errno = EBADF;
   return(0);
}

